@@ -1,5 +1,5 @@
 /*
- *   stunnel       Universal SSL tunnel
+ *   stunnel       TLS offloading and load-balancing proxy
  *   Copyright (C) 1998-2015 Michal Trojnara <Michal.Trojnara@mirt.net>
  *
  *   This program is free software; you can redistribute it and/or modify it
@@ -87,7 +87,7 @@ ICON_IMAGE load_icon_file(const char *name) {
 
 /**************************************** client callbacks */
 
-void ui_new_chain(const int section_number) {
+void ui_new_chain(const unsigned section_number) {
     (void)section_number; /* skip warning about unused parameter */
 }
 
