@@ -41,6 +41,8 @@
 /**************************************** prototypes */
 
 /* verify initialization */
+NOEXPORT void set_client_CA_list(SERVICE_OPTIONS *section);
+NOEXPORT void auth_warnings(SERVICE_OPTIONS *);
 NOEXPORT int load_file_lookup(X509_STORE *, char *);
 NOEXPORT int add_dir_lookup(X509_STORE *, char *);
 
@@ -69,19 +71,16 @@ NOEXPORT void log_time(const int, const char *, ASN1_TIME *);
 /**************************************** verify initialization */
 
 int verify_init(SERVICE_OPTIONS *section) {
-    STACK_OF(X509_NAME) *ca_dn;
-    char *ca_name;
-    int i, verify_mode=SSL_VERIFY_PEER;
-
-    if(section->verify_level<0)
-        return 0; /* OK - no certificate verification */
+    int verify_mode=0;
 
+    /* revocation store initialization */
     section->revocation_store=X509_STORE_new();
     if(!section->revocation_store) {
         sslerror("X509_STORE_new");
         return 1; /* FAILED */
     }
 
+    /* CA initialization */
     if(section->ca_file) {
         if(!SSL_CTX_load_verify_locations(section->ctx,
                 section->ca_file, NULL)) {
@@ -93,20 +92,9 @@ int verify_init(SERVICE_OPTIONS *section) {
         /* revocation store needs CA certificates for CRL validation */
         if(load_file_lookup(section->revocation_store, section->ca_file))
             return 1; /* FAILED */
-        /* trusted CA names sent to clients for client cert selection */
-        if(!section->option.client) { /* only performed on server */
-            s_log(LOG_DEBUG, "Client CA list: %s",
-                section->ca_file);
-            ca_dn=SSL_load_client_CA_file(section->ca_file);
-            for (i=0; i<sk_X509_NAME_num(ca_dn); ++i) {
-                ca_name=X509_NAME2text(sk_X509_NAME_value(ca_dn, i));
-                s_log(LOG_INFO, "Client CA: %s", ca_name);
-                str_free(ca_name);
-            }
-            SSL_CTX_set_client_CA_list(section->ctx, ca_dn);
-        }
+        if(!section->option.client) /* only performed on server */
+            set_client_CA_list(section);
     }
-
     if(section->ca_dir) {
         if(!SSL_CTX_load_verify_locations(section->ctx,
                 NULL, section->ca_dir)) {
@@ -115,29 +103,72 @@ int verify_init(SERVICE_OPTIONS *section) {
             sslerror("SSL_CTX_load_verify_locations");
             return 1; /* FAILED */
         }
-        s_log(LOG_DEBUG, "Verify directory set to %s", section->ca_dir);
+        s_log(LOG_INFO, "Verify directory set to %s", section->ca_dir);
         /* revocation store needs CA certificates for CRL validation */
         add_dir_lookup(section->revocation_store, section->ca_dir);
     }
 
+    /* CRL initialization */
     if(section->crl_file)
         if(load_file_lookup(section->revocation_store, section->crl_file))
             return 1; /* FAILED */
-
     if(section->crl_dir) {
         section->revocation_store->cache=0; /* don't cache CRLs */
         add_dir_lookup(section->revocation_store, section->crl_dir);
     }
 
+    /* verify callback setup */
+    if(section->verify_level>=0)
+        verify_mode|=SSL_VERIFY_PEER;
     if(section->verify_level>=2 && !section->redirect_addr.names)
         verify_mode|=SSL_VERIFY_FAIL_IF_NO_PEER_CERT;
     SSL_CTX_set_verify(section->ctx, verify_mode, verify_callback);
+    auth_warnings(section);
 
-    if(section->ca_dir && section->verify_level>=3)
-        s_log(LOG_INFO, "Peer certificate location %s", section->ca_dir);
     return 0; /* OK */
 }
 
+/* trusted CA names sent to clients for client cert selection */
+NOEXPORT void set_client_CA_list(SERVICE_OPTIONS *section) {
+    STACK_OF(X509_NAME) *ca_dn;
+    char *ca_name;
+    int i;
+
+    s_log(LOG_DEBUG, "Client CA list: %s", section->ca_file);
+    ca_dn=SSL_load_client_CA_file(section->ca_file);
+    for(i=0; i<sk_X509_NAME_num(ca_dn); ++i) {
+        ca_name=X509_NAME2text(sk_X509_NAME_value(ca_dn, i));
+        s_log(LOG_INFO, "Client CA: %s", ca_name);
+        str_free(ca_name);
+    }
+    SSL_CTX_set_client_CA_list(section->ctx, ca_dn);
+}
+
+/* issue warnings on insecure/missing authentication */
+NOEXPORT void auth_warnings(SERVICE_OPTIONS *section) {
+#ifndef OPENSSL_NO_PSK
+    if(section->psk_keys)
+        return;
+#endif /* !defined(OPENSSL_NO_PSK) */
+    if(!section->option.client) /* servers don't require authentication */
+        return;
+    if(section->verify_level<2) {
+        s_log(LOG_WARNING,
+            "Service [%s] needs authentication to prevent MITM attacks",
+            section->servname);
+        return;
+    }
+    if(section->verify_level>=3) /* levels>=3 don't rely on PKI */
+        return;
+#if OPENSSL_VERSION_NUMBER>=0x10002000L
+    if(section->check_email || section->check_host || section->check_ip)
+        return;
+#endif /* OPENSSL_VERSION_NUMBER>=0x10002000L */
+    s_log(LOG_WARNING,
+        "Service [%s] uses \"verify = 2\" without subject checks",
+        section->servname);
+}
+
 NOEXPORT int load_file_lookup(X509_STORE *store, char *name) {
     X509_LOOKUP *lookup;
 
